@@ -1,4 +1,4 @@
-/* $Id: thunar-gdk-extensions.h 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2003-2007 Benedikt Meurer <benny@xfce.org>
  *
@@ -25,9 +25,7 @@
 G_BEGIN_DECLS;
 
 GdkScreen *thunar_gdk_screen_open       (const gchar *display_name,
-                                         GError     **error) G_GNUC_INTERNAL;
-
-GdkScreen *thunar_gdk_screen_get_active (void) G_GNUC_INTERNAL;
+                                         GError     **error);
 
 G_END_DECLS;
 
